-- Main module to generate the initial Spicey application

import Directory
import List(last)
import System(system, getArgs)
import SpiceyScaffolding
import Distribution

banner = line ++ "Spicey Web Framework (Version of 23/09/10)\n" ++ line
 where line = take 70 (repeat '-') ++ "\n"

data FileMode = Exec | NoExec

setFileMode fmode filename =
  if fmode==Exec then system ("chmod +x \"" ++ filename ++ "\"") >> done
                 else done

data DirTree =
   Directory String [DirTree] -- a directory to be created
 | ResourceFile FileMode String   -- a file to be copied from resource directory
 | ResourcePatchFile FileMode String (String->String) -- file to be copied from
      -- resource directory where its contents is patched by the given function
 | GeneratedFromERD (String -> String -> String -> IO ()) -- takes function to generate code from ERD specification

structure = 
  Directory "." [
    ResourceFile NoExec "README.txt",
    ResourcePatchFile NoExec "Makefile" replacePakcsDir,
    Directory "scripts" [ 
      ResourcePatchFile Exec "deploy.sh"  replacePakcsDir,
      ResourcePatchFile Exec "load.sh"    replacePakcsDir,
      ResourcePatchFile Exec "run.sh"     replacePakcsDir,
      ResourcePatchFile Exec "compile.sh" replacePakcsDir],
    Directory "system" [
      ResourceFile NoExec "Main.curry",
      ResourceFile NoExec "Spicey.curry",
      ResourceFile NoExec "Routes.curry",
      ResourceFile NoExec "Session.curry",
      ResourceFile NoExec "Authorization.curry",
      ResourceFile NoExec "Authentication.curry",
      ResourceFile NoExec "Processes.curry" ],
    Directory "views" [
      ResourceFile NoExec "SpiceySystemView.curry",
      GeneratedFromERD createViewsForTerm,
      GeneratedFromERD createHtmlHelpersForTerm ],
    Directory "controllers" [
      ResourceFile NoExec "SpiceySystemController.curry",
      GeneratedFromERD createControllersForTerm ],
    Directory "models" [
      GeneratedFromERD createModelsForTerm,
      ResourceFile NoExec "ERD.curry",
      ResourceFile NoExec "ERDGeneric.curry" ],
    Directory "config" [
      ResourceFile NoExec "UserProcesses.curry",
      GeneratedFromERD createRoutesForTerm ],
    Directory "public" [
      ResourceFile NoExec "index.html",
      ResourceFile NoExec "favicon.ico",
      Directory "css" [
        ResourceFile NoExec "style.css"
      ],
      Directory "images" [
        ResourceFile NoExec "spicey-logo.png",
        ResourceFile NoExec "text.png",
        ResourceFile NoExec "time.png",
        ResourceFile NoExec "number.png",
        ResourceFile NoExec "foreign.png"
      ]
    ]
  ]

resourceDirectoryLocal = "resource_files" -- script directory gets prepended

-- Replace every occurrence of "XXXPAKCSXXX" by Distribution.installDir
replacePakcsDir :: String -> String
replacePakcsDir [] = []
replacePakcsDir (c:cs)
  | c=='X' && take 10 cs == "XXPAKCSXXX"
    = installDir ++ replacePakcsDir (drop 10 cs)
  | otherwise = c : replacePakcsDir cs

prependPath :: String -> String -> String
prependPath path name  | last path == '/'  = path ++ name
                       | otherwise         = path ++ "/" ++ name

  
copyFileLocal :: FileMode -> String -> String -> String -> IO ()
copyFileLocal fmode path generator_path filename = do
  let infile  = prependPath (prependPath generator_path resourceDirectoryLocal)
                            filename
  let outfile = prependPath path filename
  system $ "cp \"" ++ infile ++ "\" \"" ++ outfile ++ "\""
  setFileMode fmode outfile

-- checks if given path exists (file or directory) and executes
-- given action if not
ifNotExistsDo :: String -> IO () -> IO ()
ifNotExistsDo path cmd = do
  fileExists <- doesFileExist path
  dirExists  <- doesDirectoryExist path
  if fileExists || dirExists
    then putStrLn ("Skipping " ++ path ++ " ...")
    else cmd

createStructure :: String -> String -> String -> String -> DirTree -> IO ()
createStructure target_path generator_path _ _ (ResourceFile fmode filename) =
  let full_path = prependPath target_path filename
   in ifNotExistsDo full_path
         (putStrLn ("Creating file " ++ full_path ++ " ...") >>
          copyFileLocal fmode target_path generator_path filename)
      
createStructure target_path generator_path _ _
                (ResourcePatchFile fmode filename f) =
  let full_path = prependPath target_path filename
   in ifNotExistsDo full_path $ do
         putStrLn ("Creating file " ++ full_path ++ " ...")
         cnt <- readFile (prependPath (prependPath generator_path
                                                   resourceDirectoryLocal)
                                       filename)
         let outfile = prependPath target_path filename
         writeFile outfile (f cnt)
         setFileMode fmode outfile

createStructure target_path generator_path term_path db_path
                (Directory dirname subtree) = do
  let full_path = prependPath target_path dirname
  ifNotExistsDo full_path (putStrLn ("Creating directory "++full_path++" ...")
                           >> createDirectory full_path)
  foldl (\a b -> a >> createStructure full_path generator_path term_path db_path b)
        done subtree
      
createStructure target_path _ term_path db_path
                (GeneratedFromERD generatorFunction) = do
  putStrLn $ "Generating from Term File " ++ term_path ++ " ..."
  generatorFunction term_path target_path db_path
    
--- The main operation to start the scaffolding
main = do
  putStrLn banner
  args <- getArgs
  if length args /= 4
    then putStrLn $
           "Please supply the path where the project should reside,\n"++
           "the path of the generator, the path to the term file,\n"++
           "and the prefix of the DB files"
    else createStructure (head args) (args!!1) (args!!2) (args!!3) structure
